package org.ten60.netkernel.mod.pds;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.netkernel.layer0.nkf.INKFExpiryFunction;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.util.GoldenThreadExpiryFunction;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.IRequest;
import org.netkernel.request.IRequestResponseFields;


public class PDSInMemoryImpl extends StandardAccessorImpl
{
	private Map<String,InMemoryResource> mTable;
	
	public PDSInMemoryImpl()
	{	this.declareThreadSafe();
		mTable = new ConcurrentHashMap<String, InMemoryResource>();
	}
	
	@Override
	public void onRequest(INKFRequestContext context) throws Exception
	{	int verb=context.getThisRequest().getVerb();
		if (verb==INKFRequestReadOnly.VERB_META)
		{	super.onMeta(context);
		}
		else
		{	onRequest(
					context.getThisRequest().getArgumentValue("zone"),
					context.getThisRequest().getArgumentValue("pds"),
					context
					);			
		}
	}

	public void onRequest(String aZone, String aKey, INKFRequestContext aHelper) throws NKFException
	{
		String key=aZone+":"+aKey;
		switch(aHelper.getThisRequest().getVerb())
		{
			case INKFRequestReadOnly.VERB_SINK:
				sink(key,aHelper);
				break;
			case INKFRequestReadOnly.VERB_SOURCE:
				source(key,aHelper);
				break;
			case INKFRequestReadOnly.VERB_EXISTS:
				exists(key,aHelper);
				break;
			case INKFRequestReadOnly.VERB_DELETE:
				delete(key,aHelper);
				break;
		}
	}	
		
	public void sink(String aKey, INKFRequestContext aHelper)
	{	INKFResponseReadOnly primary = aHelper.getThisRequest().getPrimaryAsResponse();
		boolean debug = aHelper.getKernelContext().getKernel().inDebug();
		String debugString=debug?aHelper.getThisRequest().getIdentifier():null;
		InMemoryResource resource = new InMemoryResource(primary,debugString);
		InMemoryResource existing=mTable.put(aKey, resource);
		if (existing!=null)
		{	existing.invalidate();
		}
		INKFResponse response = aHelper.createResponseFrom(null);
		response.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}

	public void source(String aKey, INKFRequestContext aHelper) throws NKFException
	{	InMemoryResource r = mTable.get(aKey);
		if (r!=null)
		{	INKFResponse response = aHelper.createResponseFrom(r.getRepresentation());
			response.setExpiry(INKFResponse.EXPIRY_FUNCTION, r.getExpiry());
			if(r.getUserMetaData()!=null)
			{	response.setHeaders(r.getUserMetaData());
			}
		}
		else
		{	//default response of null
		}
	}
	
	public void exists(String aKey, INKFRequestContext aHelper)
	{	InMemoryResource r = mTable.get(aKey);
		INKFResponse response = aHelper.createResponseFrom(r!=null && r.getRepresentation()!=null);
		INKFExpiryFunction expiry;
		if (r!=null)
		{	expiry = r.getExpiry();
		}
		else
		{	boolean debug = aHelper.getKernelContext().getKernel().inDebug();
			String debugString=debug?aHelper.getThisRequest().getIdentifier():null;
			r=new InMemoryResource(null,debugString);
			mTable.put(aKey,r);
			expiry = r.getExpiry();
		}
		response.setExpiry(INKFResponse.EXPIRY_FUNCTION, expiry);
	}
	
	public void delete(String aKey, INKFRequestContext aHelper)
	{	InMemoryResource r = mTable.remove(aKey);
		if (r!=null)
		{	r.invalidate();
		}
		INKFResponse response = aHelper.createResponseFrom(r!=null);
		response.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}
	
	
	
	private static class InMemoryResource
	{
		private final GoldenThreadExpiryFunction mExpiry;
		private final Object mRepresentation;
		private final IRequestResponseFields mUserMeta;
		
		public InMemoryResource(INKFResponseReadOnly aResponse, String aDebug)
		{	
			if (aDebug!=null)
			{	mExpiry = new GoldenThreadExpiryFunction(aDebug);
			}
			else
			{	mExpiry = new GoldenThreadExpiryFunction("PDS");
			}			
			if (aResponse!=null)
			{	mRepresentation=aResponse.getRepresentation();
				mUserMeta=aResponse.getHeaders();
			}
			else
			{	mRepresentation=null;
				mUserMeta=null;
			}
		}		
		
		public void invalidate()
		{	mExpiry.invalidate();
		}
		
		public Object getRepresentation()
		{	return mRepresentation;
		}
		
		public INKFExpiryFunction getExpiry()
		{	return mExpiry;
		}
		
		public IRequestResponseFields getUserMetaData()
		{	return mUserMeta;
		}
	}
}
